/*
 * @(#)MockDefaultToolbar.java
 *
 * Copyright (c) 2008 Jeremy Wood. All Rights Reserved.
 *
 * You have a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) You do not utilize the software in a manner
 * which is disparaging to the original author.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. THE AUTHOR SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL THE
 * AUTHOR BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 */

package com.bric.swing.toolbar;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;
import javax.swing.JSeparator;

import com.bric.swing.MockComponent;

/** This is a rendering of the default toolbar.
 * This may look different from the actual toolbar because all
 * flexible spaces are compacted to the minimum/default space
 * width.
 */
class MockDefaultToolbar extends MockComponent {

	private static final long serialVersionUID = 1L;
	
	public MockDefaultToolbar(CustomizedToolbar toolbar) {
		super(createImage(toolbar));
	}
	
	private static BufferedImage createImage(CustomizedToolbar toolbar) {
		String[] contents = toolbar.getDefaultContents();
		Insets insets = toolbar.getComponentInsets();
		Dimension[] sizes = new Dimension[contents.length];
		JComponent[] components = new JComponent[contents.length];
		
		int minWidth = 0;
		for(int a = 0; a<contents.length; a++) {
			minWidth += insets.left+insets.right;
			components[a] = toolbar.getComponent(contents[a]);
			sizes[a] = components[a].getPreferredSize();
			if(components[a] instanceof JSeparator) {
				sizes[a].height = toolbar.minimumHeight;
			}
			if(contents[a].length()>0 && contents[a].charAt(0)=='\t') {
				sizes[a].width = SpaceComponent.SPACE_COMPONENT_WIDTH;
			}
			minWidth += sizes[a].width;
		}
		
		BufferedImage bi = new BufferedImage(minWidth,toolbar.minimumHeight+insets.top+insets.bottom,BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = bi.createGraphics();

		
		g.setColor(new Color(0,0,0,80));
		g.drawRect(0, 0, bi.getWidth()-1, bi.getHeight()-1);
		
		g.translate(0,insets.top);
		for(int a = 0; a<contents.length; a++) {
			g.translate(insets.left, 0);
			int dy = toolbar.minimumHeight/2-sizes[a].height/2;
			g.translate(0, dy);
			
			if(contents[a].length()>0 && contents[a].charAt(0)=='\t') {
				SpaceComponent.paintSpace(g,false,true,sizes[a].width,sizes[a].height);
			} else if(contents[a].length()>0 && contents[a].charAt(0)=='-') {
				MacToolbarSeparatorUI.paintSeparator(g,sizes[a].width,sizes[a].height);
			} else {
				components[a].paint(g);
			}
			
			g.translate(sizes[a].width, 0);
			g.translate(insets.right, 0);
			g.translate(0, -dy);
		}
		
		g.dispose();
		return bi;
	}
}