/*
 * Copyright (c) 2010, Franziska Hufsky
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the Friedrich-Schiller-University Jena nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.unijena.bioinf.labelingdetector.ui;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import de.randgestalten.bean.AbstractBean;
import de.unijena.bioinf.labelingdetector.AlgorithmSettings;
import de.unijena.bioinf.labelingdetector.Analyzer;
import epos.ui.util.BindingUtils;
import epos.ui.util.ComponentUtils;

import javax.swing.*;

/**
 * Created by IntelliJ IDEA.
 * User: fhufsky
 * Date: Jun 30, 2010
 * Time: 3:40:35 PM
 * To change this template use File | Settings | File Templates.
 */
public class AlgorithmSettingsPanel extends AbstractBean {

    public enum Score{MassesAndIntensities("Masses and intensities", AlgorithmSettings.Score.MI), MassDiffAndIntensities("Mass differences and intensities", AlgorithmSettings.Score.MDI), Intensities("Intensities",AlgorithmSettings.Score.I);
        private String name;
        private AlgorithmSettings.Score score;

        Score(String name, AlgorithmSettings.Score score) {

            this.name = name;
            this.score = score;
        }

        public AlgorithmSettings.Score getScore() {
            return score;
        }

        @Override
        public String toString() {
            return name;
        }
    }

    private Score scoring = Score.MassesAndIntensities;
    private double scoringOffset = 0.02;
    private double normalizationOffset =0;
    private int numberOfDecimal = 2;

    public double getNormalizationOffset(){
        return normalizationOffset;
    }

    public void setNormalizationOffset(double normalizationOffset){
        firePropertyChange("normalizationOffset", this.normalizationOffset, this.normalizationOffset=normalizationOffset);
    }

    public int getNumberOfDecimal() {
        return numberOfDecimal;
    }

    public void setNumberOfDecimal(int numberOfDecimal) {
        firePropertyChange("numberOfDecimal", this.numberOfDecimal, this.numberOfDecimal = numberOfDecimal);
    }

    public double getScoringOffset() {
        return scoringOffset;
    }

    public void setScoringOffset(double scoringOffset) {
        firePropertyChange("scoringOffset", this.scoringOffset, this.scoringOffset = scoringOffset);
    }


    public Score getScoring() {
        return scoring;
    }

    public void setScoring(Score scoring) {
        firePropertyChange("scoring", this.scoring, this.scoring = scoring);
    }

    public JComponent getComponent(){

        // components
        JComboBox scoreBox = ComponentUtils.createComboBox(Score.values(), Score.MassesAndIntensities);
        JSpinner scoringSpinner = ComponentUtils.createSpinner(0.02,0,1,0.01);
        JSpinner normSpinner = ComponentUtils.createSpinner(0,0,1,0.01);
        JSpinner decimalSpinner = ComponentUtils.createSpinner(2,0,3,1);


        // layout
        DefaultFormBuilder builder = ComponentUtils.formBuilder();
        builder.append("Scoring function", scoreBox);
        builder.append("Renormalize with peaks higher than", normSpinner);
        builder.append("Score only peaks higher than", scoringSpinner);
        builder.append("Number of decimals", decimalSpinner);


        // logic
        BindingUtils.bindReadWrite(this, "scoringOffset", scoringSpinner, "value").bind();
        BindingUtils.bindReadWrite(this, "scoring", scoreBox, "selectedItem").bind();
        BindingUtils.bindReadWrite(this, "numberOfDecimal", decimalSpinner, "value").bind();
        BindingUtils.bindReadWrite(this, "normalizationOffset", normSpinner, "value").bind();

        return builder.getPanel();
    }

    public static void main(String[] args) {
        JFrame f = new JFrame();
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setContentPane(new MoleculeSettingsPanel().getComponent());
        f.pack();
        f.setVisible(true);
    }
}
