/*
 * Copyright (c) 2010, Franziska Hufsky
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the Friedrich-Schiller-University Jena nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.unijena.bioinf.labelingdetector.ui;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import de.randgestalten.bean.AbstractBean;
import epos.ui.util.BindingUtils;
import epos.ui.util.ComponentUtils;

import javax.swing.*;

/**
 * Created by IntelliJ IDEA.
 * User: fhufsky
 * Date: Jun 30, 2010
 * Time: 4:00:01 PM
 * To change this template use File | Settings | File Templates.
 */
public class MSSettingsPanel extends AbstractBean {

    private double alpha1=3;
	private double alpha0=1.5*alpha1;
	private double beta1=10;
	private double beta0=90;

    public double getAlpha0() {
        return alpha0;
    }

    public void setAlpha0(double alpha0) {
        firePropertyChange("alpha0", this.alpha0, this.alpha0=alpha0);
    }

    public double getAlpha1() {
        return alpha1;
    }

    public void setAlpha1(double alpha1) {
        firePropertyChange("alpha1",this.alpha1, this.alpha1=alpha1);
        setAlpha0(alpha1*1.5);
    }

    public double getBeta0() {
        return beta0;
    }

    public void setBeta0(double beta0) {
        firePropertyChange("beta0", this.beta0, this.beta0=beta0);
    }

    public double getBeta1() {
        return beta1;
    }

    public void setBeta1(double beta1) {
        firePropertyChange("beta1", this.beta1, this.beta1=beta1);
    }

    public JComponent getComponent(){

        // components
        JSpinner alpha1Spinner = ComponentUtils.createSpinner(3., 1, 100, 1);
        JSpinner alpha0Spinner = ComponentUtils.createSpinner(1.5*alpha1,1,100,1);
        JSpinner beta1Spinner = ComponentUtils.createSpinner(10.,1,100,1);
        JSpinner beta0Spinner = ComponentUtils.createSpinner(90.,1,100,1);

        // layout
        DefaultFormBuilder builder = ComponentUtils.formBuilder();
      
        builder.append("alpha1 (full Int) (ppm)", alpha1Spinner);
        builder.append("alpha0 (low Int) (ppm)", alpha0Spinner);
        builder.append("beta1 (full Int) (%)", beta1Spinner);
        builder.append("beta0 (low Int) (%)", beta0Spinner);

        // logic
        BindingUtils.bindReadWrite(this, "alpha1",alpha1Spinner, "value").bind();
        BindingUtils.bindReadWrite(this, "alpha0", alpha0Spinner, "value").bind();
        BindingUtils.bindReadWrite(this, "beta1", beta1Spinner, "value").bind();
        BindingUtils.bindReadWrite(this, "beta0", beta0Spinner, "value").bind();

        return builder.getPanel();
    }

    public static void main(String[] args) {
        JFrame f = new JFrame();
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setContentPane(new MSSettingsPanel().getComponent());
        f.pack();
        f.setVisible(true);
    }
}
