/*
 * Copyright (c) 2010, Franziska Hufsky
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the Friedrich-Schiller-University Jena nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.unijena.bioinf.labelingdetector.ui.wizard;

import de.unijena.bioinf.labelingdetector.Spectrum;
import de.unijena.bioinf.labelingdetector.ui.AnalysedSpectrum;
import de.unijena.bioinf.labelingdetector.ui.Utils;
import epos.ui.util.ComponentUtils;
import epos.ui.wizard.AbstractWizardPanel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.List;

/**
 *
 *
 * @author Thasso Griebel (thasso.griebel@gmail.com)
 */
public class CopyFromclipboardPanel extends AbstractWizardPanel{
    private JTextArea textArea;
    private JComponent main;
    private java.util.List<AnalysedSpectrum> spectra;

    public CopyFromclipboardPanel() {
        super("Enter Spectra", "Please enter spectra of the following format:\nPeak\tIntensity\nPeak\tIntensity\n...\nSeveral spectra can be added separated by ***");
    }

    public JComponent getComponent() {
        if(main == null){
            textArea = ComponentUtils.createTextArea();
            textArea.addKeyListener(new KeyAdapter() {
                public void keyReleased(KeyEvent e) {
                    CopyFromclipboardPanel.this.wizard.validate();
                }
            });
            main = new JPanel(new BorderLayout());
            JScrollPane scroll = ComponentUtils.createScrollPane(textArea);            
            main.add(scroll, BorderLayout.CENTER);
        }
        return main;
    }

    @Override
    public String validate() {
        if(textArea.getText().trim().length() == 0){
            return "Please enter at least one spectrum!";
        }
        try {
            spectra = Utils.readInputSpectra(textArea.getText());            
        } catch (Exception e) {         
            return "Error while creating spectra! Check format!";
        }
        return super.validate();        
    }

    public List<AnalysedSpectrum> getSpectra() {
        return spectra;
    }
}

